#include "scrollbarnode.h"

#include "scene/meshnode.h"

#include "SparrowRenderer/mesh.h"
#include "SparrowRenderer/phongmaterial.h"
#include "sparrowshell/sparrowshell.h"

ScrollBarNode::ScrollBarNode(glm::vec2 dimension):
    m_dimension(dimension),
    m_bar_position(glm::vec2(0)),
    m_bar_dimension(glm::vec2(0))
{
    Mesh* mesh = new Mesh();
    mesh->addRectangle2D(glm::vec2(0),m_dimension);
    PhongMaterial* mat = new PhongMaterial();
    mat->diffuse = glm::vec3(0,0,0.5);
    mat->m_opacity = 0.8;
    mesh->setMaterial(mat);
    mesh->setDepth(SparrowShell::SHELL_DEPTH+1);
    mesh->initGL();
    m_bar = new MeshNode(mesh,false);
    addChild(m_bar);
}

void ScrollBarNode::update()
{
    GUINode::update();
    if(m_barResized)
    {
        m_bar->resize2D(m_bar_dimension,m_last_bar_dimension);
        m_barResized = false;
    }
    if(m_barMoved)
    {
        m_bar->moveTo2D(m_bar_position);
        m_barMoved = false;
    }
}

void ScrollBarNode::moveBar(glm::vec2 position)
{
    m_bar_position = position;
    m_barMoved = true;
}

void ScrollBarNode::resizeBar(glm::vec2 dimension)
{
    m_last_bar_dimension = m_bar_dimension;
    m_bar_dimension = dimension;
    m_barResized = true;
}

glm::vec2 ScrollBarNode::getDimension()
{
    return m_dimension;
}
